package lsedit;

import java.awt.Color;
import java.io.IOException;
import java.io.PrintStream;

import javax.swing.undo.UndoableEdit;

public class EntityClass extends LandscapeClassObject /* extends LandscapeObject3D extends LandscapeObject */ 
{
	class UpdateImage extends MyUndoableEdit implements UndoableEdit
	{
		EntityClass			 m_o;
		int                  m_old;
		int                  m_new;

		UpdateImage(EntityClass o, int value)
		{
			m_o   = o;
			m_old = o.getImage();
			m_new = value;
			if (logEdit(this)) {
				o.setImage(value);
		}	}

		public String getPresentationName() 
		{
			return m_o + " Image " + m_o.getImageName(m_new);
		}

		public void undo()
		{
			m_o.setImage(m_old);
		}

		public void redo()
		{
			m_o.setImage(m_new);
	}	}

	public static final String ENTITY_BASE_CLASS_ID		= "$ENTITY";

	public static final int ENTITY_STYLE_NONE           = -1;
	public static final int ENTITY_STYLE_3DBOX			= 0;
	public static final int ENTITY_STYLE_2DBOX			= 1;
	public static final int ENTITY_STYLE_FILE			= 2;
	public static final int ENTITY_STYLE_DISK			= 3;
	public static final int ENTITY_STYLE_FOLDER			= 4;
	public static final int ENTITY_STYLE_SOURCEOBJ		= 5;
	public static final int ENTITY_STYLE_CLASS			= 6;
	public static final int ENTITY_STYLE_GROUP			= 7;
	public static final int ENTITY_STYLE_LABELLED_GROUP = 8;
	public static final int ENTITY_STYLE_TRIANGLE_UP    = 9;
	public static final int ENTITY_STYLE_TRIANGLE_DOWN  = 10;
	public static final int ENTITY_STYLE_ROMBUS_RIGHT   = 11;
	public static final int ENTITY_STYLE_ROMBUS_LEFT    = 12;
	public static final int ENTITY_STYLE_DIAMOND        = 13;
	public static final int ENTITY_STYLE_PENTAGON       = 14;
	public static final int ENTITY_STYLE_HEXAGON        = 15;
	public static final int ENTITY_STYLE_OCTAGON        = 16;
	public static final int ENTITY_STYLE_PAPER          = 17;

	public static final String[] styleName =
		{ 
				"3D Box", 
				"2D Box", 
				"File", 
				"Disk", 
				"Folder", 
				"Source Object", 
				"Class", 
				"Group", 
				"Labelled Group",
				"Triangle",
				"Inverted Triangle",
				"Rombus",
				"Inverted Rombus",
				"Diamond",
				"Pentagon",
				"Hexagon",
				"Octagon",
				"Paper"
		};


	public static final int ENTITY_IMAGE_NONE           =  -1;
	public static final int ENTITY_IMAGE_ACTOR          =   0;
	public static final int ENTITY_IMAGE_OVAL           =   1;
	public static final int ENTITY_IMAGE_X              =   2;

	public static final String[] imageName =
		{ 
				"Actor",
				"Oval",
				"X"
		};

	private boolean	m_shown = true;

	// ----------------
	// Exported methods
	// ----------------

	// Constructor 

	public EntityClass(String id, int nid, Diagram dg) 
	{
		setId(id);
		setNid(nid); 
		setLabel(id); 
		setLandscapeObjectDiagram(dg);
	}

	public static String getEntityStyleName(int style) 
	{
		if (style < 0 || style >= EntityClass.styleName.length) {
			return "";
		}
		return styleName[style];
	}

	public String getStyleName(int style)
	{
		return EntityClass.getEntityStyleName(style);
	}

	public void updateImage(int value)
	{
		if (value != m_image) {
			new UpdateImage(this, value);
	}	}

	public static String getImageName(int image) 
	{
		if (image < 0 || image >= EntityClass.imageName.length) {
			return "";
		}
		return imageName[image];
	}

	public void setShown(boolean value)
	{
		m_shown = value;
	}

	public boolean isShown()
	{
		return(m_shown);
	}

	public void writeAttributes(PrintStream ps) throws IOException {

		ps.print(qt(getId()) + " {\n");
		super.writeAttributes(ps);
		ps.print("}\n\n"); 
	}

	public EntityInstance newEntity(String id) 
	{
		EntityInstance e = new EntityInstance(this, id);
		makeInstanceOfUs(e);
		return e;
	}

	// The routines that follow hide the complexity of getting/setting attribute values
	// from EditAttributes

	public int getPrimaryAttributeCount()
	{
		return(8);
	}

	public String getLsAttributeNameAt(int index)
	{
		String	name;

		switch (index) {
		case 0:
			name  = "id";
			break;
		case 1:
			name  = CLASSLABEL_ID;
			break;
		case 2:
			name  = CLASSDESC_ID;
			break;
		case 3:
			name  = CLASSSTYLE_ID;
			break;
		case 4:	
			name  = CLASSIMAGE_ID;
			break;
		case 5:
			name  = COLOUR_ID;
			break;
		case 6:
			name  = LABEL_COLOUR_ID;
			break;
		case 7:
			name  = OPEN_COLOUR_ID;
			break;
		default:
			name  = super.getLsAttributeNameAt(index);
		}
		return(name);
	}

	public Object getLsAttributeValueAt(int index)
	{
		Object	value;

		switch (index) {
		case 0:
			value = getId();
			break;
		case 1:
			value = getLabel();
			break;
		case 2:
			value = getDescription();
			break;
		case 3:
			value = new Integer(getStyle());
			break;
		case 4:
			value = new Integer(getImage());
			break;
		case 5:
			value = getObjectColor();
			break;
		case 6:
			value = getLabelColor();
			break;
		case 7:
			value = getColorWhenOpen();
			break;
		default:
			value = super.getLsAttributeValueAt(index);
		}
		return(value);
	}

	public void setAttributeValueAt(int index, Object value)
	{
		switch (index) {
		case 0:
			setId((String) value);
			break;
		case 1:
			updateLabel((String) value);
			break;
		case 2:
			updateDescription((String) value);
			break;
		case 3:
			updateStyle(((Integer) value).intValue());
			break;
		case 4:
			updateImage(((Integer) value).intValue());
			break;
		case 5:
			updateObjectColor((Color) value);
			break;
		case 6:
			updateLabelColor((Color) value);
			break;
		case 7:
			updateColorWhenOpen((Color) value);
			break;
		default:
			super.setAttributeValueAt(index, value);
	}	}

	// Need to know the type in cases where value might be null
	// For example with some colors

	public int getLsAttributeTypeAt(int index)
	{
		int		ret;
		
		switch (index) {
		case 0:
		case 1:
			ret = Attribute.STRING_TYPE;
			break;
		case 2:
			ret = Attribute.TEXT_TYPE;
			break;
		case 3:
			ret = Attribute.ENTITY_STYLE_TYPE;
			break;
		case 4:
			ret = Attribute.ENTITY_IMAGE_TYPE;
			break;
		case 5:
		case 6:
		case 7:
			ret = Attribute.COLOR_OR_NULL_TYPE;
			break;
		default:
			ret = super.getLsAttributeTypeAt(index);
		}
		return(ret);
	}
}



